#!/usr/bin/env python
# encoding: utf-8
#
# ** header v3.0
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2018 Research Group Biomedical Physics,
# Max-Planck-Institute for Dynamics and Self-Organization Göttingen
# Copyright (C) 2019 Henrik tom Wörden
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# ** end header
"""does something"""

import argparse
import re
from argparse import RawTextHelpFormatter
from copy import deepcopy
from datetime import datetime

import caosdb as db
from caosadvancedtools.cache import Cache
from caosdb.exceptions import TransactionError

ENTITIES = {}


def get_value(prop):
    """returns the value of a Property

    If the value is an Entity itself, its id is returned instead
    """

    if not isinstance(prop.value, db.Entity):
        return prop.value
    else:
        return prop.value.id


def get_entity(name):
    """implements a mini cache of RecordTypes

    If and only if a RecordType (identified by its name) is not in the
    dictionary, it is  retrieved from CaosDB.
    """

    if name not in ENTITIES:
        ent = db.Entity(name=name)
        ent.retrieve()
        ENTITIES[name] = ent

    return ENTITIES[name]


class AbstractCFood(object):
    # TODO restructure this class such that no instance is needed to check for
    # a match
    # instances shall be used to keep track of a match; i.e. entities can be
    # object variable
    def __init__(self, pattern, use_cache=False, access=lambda x: x):
        self.pattern = re.compile(pattern)
        self.use_cache = use_cache
        self.access = access

        if self.use_cache:
            self.identifiable_cache = Cache()

    def treat_match(self, crawled_file, match):
        print(crawled_file)

        entities = self.create_identifiables(crawled_file, match)

        for key, identifiable in entities.items():

            if identifiable is None:
                print("THIS IS STRANGE")

                continue
            existing = None

            print("Looking for \n", identifiable)

            if self.use_cache:
                identifiable_cache = Cache()
                identifier = Cache.hash_entity(identifiable)
                cached_id = self.identifiable_cache.check_existing(identifier)

                # retrieve entity for the cached id

                if cached_id is not None:
                    existing = db.execute_query("FIND {}".format(cached_id),
                                                unique=True)
                    print("Found Entity in cache; Id:", cached_id)

            # Nothing in cache or cache not used. Check in CaosDB

            if existing is None:
                existing = AbstractCFood.find_existing(identifiable)

            # No record matching the identifiable was found. Insert the record

            if existing is None:
                identifiable.insert()
                entities[key] = identifiable
            else:
                entities[key] = existing

            print("Got\n", identifiable)

            if self.use_cache:
                print("cid", cached_id)

            if self.use_cache and cached_id is None:
                identifiable_cache.insert(identifier, entities[key].id)

        self.update_identifiables(entities, crawled_file, match)

    def create_identifiables(self, crawled_file, match):
        raise NotImplementedError()

    def update_identifiables(self, entities, crawled_file, match):
        raise NotImplementedError()

    @staticmethod
    # move to api?
    def set_parents(entity, names):
        entity.parents.clear()

        for n in names:
            entity.add_parent(get_entity(n))

    @staticmethod
    # move to api?
    def remove_property(entity, prop):
        # TODO only do something when it is necessary?

        if isinstance(prop, db.Entity):
            name = prop.name
        else:
            name = prop

        while entity.get_property(name) is not None:
            entity.remove_property(name)

    @staticmethod
    # move to api?
    def set_property(entity, prop, value, datatype=None):
        AbstractCFood.remove_property(entity, prop)

        if datatype is not None:
            entity.add_property(prop, value, datatype=datatype)
        else:
            entity.add_property(prop, value)

    @staticmethod
    def find_existing(entity):
        """searches for an entity that matches the identifiable in CaosDB

        Characteristics of the identifiable like, properties, name or id are
        used for the match.
        """

        if entity.name is None:
            # TODO multiple parents are ignored! Sufficient?
            query_string = "FIND Record " + entity.get_parents()[0].name
            query_string += " with " + " and ".join(
                ["'" + p.name + "'='"
                 + str(get_value(p)) + "'" for p in entity.get_properties()])
        else:
            query_string = "FIND '{}'".format(entity.name)

        print(query_string)
        q = db.Query(query_string)
        # the identifiable should identify an object uniquely. Thus the query
        # is using the unique keyword
        try:
            r = q.execute(unique=True)
        except TransactionError as er:
            r = None

        if r is not None:
            print("Found Entity with id:", r.id)
        else:
            print("Did not find an existing entity")

        return r


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=RawTextHelpFormatter)

    return parser


if __name__ == "__main__":
    parser = get_parser()
    args = parser.parse_args()
